# STR116
The `str116.py` module controls an STR116 or STR008 relay board.

## Setup
You need to have your board connected to a serial port. This may differ from board to board, and from os to os. Using Raspbian on a raspberry pi, we use the serial port `'/dev/ttyAMA0'`, address `2`.

```python
from brewkit.str116 import STR116

# Example
port = '/dev/ttyAMA0'
address = 2

device = STR116(port, address)
```
Of course, you can do this from the python CLI as well. That might be more useful if you just need to flip a relay

## Usage
```python
board = STR116('/dev/ttyAMA0', 2)

# Get relay state
board.relay(3)  # 0 or 1

# Set relay state
board.relay(3, 1) # Turn it on
board.relay(3, 0) # Turn it off
```

## Errors
If something went wrong during setup and the board couldn't be detected, you'll get an `InvalidHardwareError` like this:
```
>>> dev = STR116('/not/a/port', 2)
[E 180911 20:59:44 str116:17] Invalid Hardware for STR116 using port /not/a/port and address 2
Traceback (most recent call last):
  File "<stdin>", line 1, in <module>
  File "brewkit/app/str116.py", line 18, in __init__
    raise InvalidHardwareError("Invalid Hardware for STR116 using port %s andaddress %s" % (self.port, self.address))
brewkit.app.exceptions.InvalidHardwareError: Invalid Hardware for STR116 using port /not/a/port and address 2
```

If you try to access a relay that doesn't exist (our board has 16), you'll see this:
```
>>> board.relay(234)
[E 180911 21:20:09 str116:45] Relay couldn't be found, see error below
[E 180911 21:20:09 str116:46] invalid literal for int() with base 10: ''
    Traceback (most recent call last):
      File "brewkit/app/str116.py", line 43, in relay
        return int(relaystatus[relay_num*2:relay_num*2+2])
    ValueError: invalid literal for int() with base 10: ''
Traceback (most recent call last):
  File "<stdin>", line 1, in <module>
  File "brewkit/app/str116.py", line 47, in relay
    raise RelayNotFoundError('Relay %s couldn\'t be found, make sure it is inrange' % relay_num)
brewkit.app.exceptions.RelayNotFoundError: Relay 234 couldn't be found, make sure it is in range
```

## Bytestrings
Bytestring breakdown:
```log
Bytestring: 55AA07140200102d77
  55  | AA  | 07 | 14 | 02 | 0010 | 2d | 77
  MA0 | MA1 | BC | CC | CN | Data | CS | MAE
Data:
    00      |   10
    Start # | # relays
Checksum: 2d
[D 180912 10:40:39 str116:74] set relay bytestring for relay 4 to state 0:
Bytestring: 55AA0817020401002677
 55  | AA  | 08 | 17 | 02 | 040100 | 26 | 77
 MA0 | MA1 | BC | CC | CN |  Data  | CS | MAE
Data: 040100
    04   |     01   |   00
 Relay # | # relays |  State
Checksum: 26
```
